@@ -1,5 +1,5 @@
 package Mail::DMARC::Report::Aggregate;
-our $VERSION = '1.20130604'; # VERSION
+our $VERSION = '1.20130605'; # VERSION
 use strict;
 use warnings;
 
@@ -170,13 +170,13 @@ Mail::DMARC::Report::Aggregate - DMARC aggregate report
 
 =head1 VERSION
 
-version 1.20130604
+version 1.20130605
 
 =head1 DESCRIPTION
 
 This class is used internally as the canonization of an aggregate report.
 
-When reports are received, the XML is parsed into a Report::Aggregate object, which then gets passed to the Report::Store to be file away. When sending DMARC reports, data is extracted from the Report::Store as an Aggregate object, assembled as XML, and sent.
+When reports are received, the XML is parsed into an L<Aggregate|Mail::DMARC::Report::Aggregate> object, which then gets passed to the Report::Store and saved. When sending DMARC reports, data is extracted from the L<Store|Mail::DMARC::Report::Store> as an Aggregate object, assembled as XML, and sent.
 
 =head1 2013 Draft Description
 
